package cachet.plugins.notifications;

/**
 * Flutter-specific
 */

import android.app.Activity;
import android.content.pm.PackageManager;
import android.widget.Toast;
import io.flutter.plugin.common.EventChannel;
import io.flutter.plugin.common.PluginRegistry.Registrar;
import io.flutter.plugin.common.EventChannel.EventSink;

/** Android-specific */
import android.content.BroadcastReceiver;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.provider.Settings;
import android.text.TextUtils;
import android.util.Log;

import org.json.JSONException;
import org.json.JSONObject;

/**
 * NotificationsPlugin
 */
public class NotificationsPlugin implements EventChannel.StreamHandler {

    public static String TAG = "NOTIFICATION_PLUGIN";

    private static final String ENABLED_NOTIFICATION_LISTENERS = "enabled_notification_listeners";
    private static final String ACTION_NOTIFICATION_LISTENER_SETTINGS = "android.settings.ACTION_NOTIFICATION_LISTENER_SETTINGS";
    private EventSink eventSink;
    private Context context;
    private static final String EVENT_CHANNEL_NAME = "notifications.eventChannel";

    /** Plugin registration. */
    public static void registerWith(Registrar registrar) {
        Log.d(TAG,"NotificationsPlugin注册");
        final EventChannel channel = new EventChannel(registrar.messenger(), EVENT_CHANNEL_NAME);
        Context context = registrar.activeContext();
        NotificationsPlugin plugin = new NotificationsPlugin(context);
        channel.setStreamHandler(plugin);
    }

    //重新打开服务
    private void toggleNotificationListenerService() {
        PackageManager pm = context.getPackageManager();
        pm.setComponentEnabledSetting(new ComponentName(context, NotificationListener.class),
                PackageManager.COMPONENT_ENABLED_STATE_DISABLED, PackageManager.DONT_KILL_APP);
        pm.setComponentEnabledSetting(new ComponentName(context, NotificationListener.class),
                PackageManager.COMPONENT_ENABLED_STATE_ENABLED, PackageManager.DONT_KILL_APP);
        Toast.makeText(context, "重新打开服务成功", Toast.LENGTH_SHORT).show();
    }

    /**
     * Plugin constructor setting the context and registering the notification service.
     */
    private NotificationsPlugin(Context context) {
        this.context = context;

        Log.d(TAG,"没有打开服务,去设置页面");
        /* Check if permission is given, if not then go to the notification settings screen. */
        if (!permissionGiven()) {
            context.startActivity(new Intent(ACTION_NOTIFICATION_LISTENER_SETTINGS));
        }else{
            Log.d(TAG,"已经设置过,重新打开服务");
            toggleNotificationListenerService();
        }

        NotificationReceiver receiver = new NotificationReceiver();
        IntentFilter intentFilter = new IntentFilter();
        intentFilter.addAction(NotificationListener.NOTIFICATION_INTENT);
        context.registerReceiver(receiver, intentFilter);
        Log.d(TAG,"注册通知接收器");
        /* Start the notification service once permission has been given. */
        Intent listenerIntent = new Intent(context, NotificationListener.class);
        context.startService(listenerIntent);
        Log.d(TAG,"开启通知监听器服务");

    }

    /** Called whenever the event channel is subscribed to in Flutter */
    @Override
    public void onListen(Object o, EventSink eventSink) {
        this.eventSink = eventSink;
        /*
          Start the notification service once permission has been given.
         */
        Intent listenerIntent = new Intent(context, NotificationListener.class);
        context.startService(listenerIntent);

    }

    /** Called whenever the event channel subscription is cancelled in Flutter */
    @Override
    public void onCancel(Object o) {
        this.eventSink = null;
    }

    /**
     * For all enabled notification listeners, check if any of them matches the package name of this application.
     * If any match is found, return true. Otherwise if no matches were found, return false.
     */
    private boolean permissionGiven() {
        String packageName = context.getPackageName();
        String flat = Settings.Secure.getString(context.getContentResolver(),
                ENABLED_NOTIFICATION_LISTENERS);
        if (!TextUtils.isEmpty(flat)) {
            String[] names = flat.split(":");
            for (String name : names) {
                ComponentName componentName = ComponentName.unflattenFromString(name);
                boolean nameMatch = TextUtils.equals(packageName, componentName.getPackageName());
                if (nameMatch) {
                    return true;
                }
            }
        }
        return false;
    }

    class NotificationReceiver extends BroadcastReceiver {
            final static String TAG = "NOTIFICATION_RECEIVER";
            @Override
        public void onReceive(Context context, Intent intent) {
            String packageName = intent.getStringExtra(NotificationListener.NOTIFICATION_PACKAGE_NAME);
            String title=intent.getStringExtra(NotificationListener.NOTIFICATION_TITLE);
            String text=intent.getStringExtra(NotificationListener.NOTIFICATION_TEXT);
            JSONObject j=new JSONObject();
            try {
                j.put("packagename",packageName);
                j.put("title", title);
                j.put("text", text);
            } catch (JSONException e) {
                e.printStackTrace();
            }
            eventSink.success(j.toString());
        }
    }
}






